package cn.qqck.kotlin.tools

import java.text.SimpleDateFormat
import java.util.Date

object Date {
    /**
     * 获取当前时间的某一天的零点对应的时间戳（毫秒）。
     *
     * 该方法默认返回当前时间对应的当日零点时间戳。如果需要推移日期，可以通过参数指定推移的天数。
     *
     * @param add 需要推移的天数，正数表示往后推移，负数表示往前推移，默认为 0（表示今天）。
     * @return 指定日期零点的时间戳，单位为毫秒。
     */
    fun day(add: Long = 0): Long {
        return (System.currentTimeMillis() + 28800000) / 86400000 * 86400000 - 28800000 + add * 86400000
    }
}

private val format_yMdHms = SimpleDateFormat("yyyy-MM-dd HH:mm:ss")
private val format_yMd = SimpleDateFormat("yyyy-MM-dd")
private val format_Hms = SimpleDateFormat("HH:mm:ss")
private val format_Hm = SimpleDateFormat("HH:mm")

/**
 * 将当前日期对象格式化为字符串，格式为“年-月-日 时:分:秒”。
 *
 * @return 格式化后的日期时间字符串。
 */
fun Date.yMdHms(): String = format_yMdHms.format(this)

/**
 * 返回日期的格式化字符串，格式为 "yyyy-MM-dd"。
 *
 * @return 一个表示当前日期的格式化字符串。
 */
fun Date.yMd(): String = format_yMd.format(this)

/**
 * 将当前日期对象格式化为 "时:分:秒" (HH:mm:ss) 格式的字符串。
 *
 * @return 格式化后的字符串，表示日期的时分秒部分。
 */
fun Date.Hms(): String = format_Hms.format(this)

/**
 * 根据当前日期对象格式化为"小时:分钟"的字符串表示（24小时制）。
 *
 * @return 返回格式化后的时间字符串，格式为"HH:mm"。
 */
fun Date.Hm(): String = format_Hm.format(this)

/**
 * 将当前时间戳（以毫秒为单位）格式化为字符串，格式为“年-月-日 时:分:秒”。
 *
 * @return 格式化后的日期时间字符串。
 */
fun Long.yMdHms(): String = Date(this).yMdHms()

/**
 * 将当前时间戳（以毫秒为单位）格式化为字符串，格式为 "yyyy-MM-dd"。
 *
 * @return 一个表示当前日期的格式化字符串。
 */
fun Long.yMd(): String = Date(this).yMd()

/**
 * 将当前时间戳（以毫秒为单位）格式化为 "时:分:秒" (HH:mm:ss) 格式的字符串。
 *
 * @return 格式化后的字符串，表示日期的时分秒部分。
 */
fun Long.Hms(): String = Date(this).Hms()

/**
 * 将当前时间戳（以毫秒为单位）格式化为"小时:分钟"的字符串表示（24小时制）。
 *
 * @return 返回格式化后的时间字符串，格式为"HH:mm"。
 */
fun Long.Hm(): String = Date(this).Hm()
