package cn.qqck.kotlin.tools

private val m_hexLittle = charArrayOf('0', '1', '2', '3', '4', '5', '6', '7', '8', '9', 'a', 'b', 'c', 'd', 'e', 'f')
private val m_hexUpper = charArrayOf('0', '1', '2', '3', '4', '5', '6', '7', '8', '9', 'A', 'B', 'C', 'D', 'E', 'F')

/**
 * 将当前字节数组转换为对应的十六进制字符串表示。
 *
 * @param upper 是否使用大写字母表示十六进制字符，默认为false（小写字母）。
 * @return 转换后的十六进制字符串。如果字节数组为空，则返回空字符串。
 */
fun ByteArray.hex(upper: Boolean = false): String {
    if (this.isEmpty()) return ""
    val j_str = StringBuilder(this.size * 2) // 一个字节对应两个16进制数，所以长度为字节数组乘2
    if (upper) {
        for (j_byte in this) {
            j_str.append(m_hexUpper[j_byte.toInt() ushr 4 and 0xf])
            j_str.append(m_hexUpper[j_byte.toInt() and 0xf])
        }
    } else {
        for (j_byte in this) {
            j_str.append(m_hexLittle[j_byte.toInt() ushr 4 and 0xf])
            j_str.append(m_hexLittle[j_byte.toInt() and 0xf])
        }
    }
    return j_str.toString()
}

private val reverseHexTable = intArrayOf(
    255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255,
    255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255,
    255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255,
    0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 255, 255, 255, 255, 255, 255, 255, 10, 11, 12, 13,
    14, 15, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255,
    255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 10, 11, 12, 13, 14,
    15, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255,
    255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255,
    255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255,
    255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255,
    255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255,
    255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255,
    255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255,
    255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255,
    255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255,
    255, 255, 255, 255, 255, 255, 255, 255, 255, 255
)

/**
 * 将当前字符串解析为十六进制格式并返回对应的字节数组。
 *
 * @param replace 是否替换字符串中的空白字符（包括空格、回车、换行和制表符）。默认为 false。
 * @return 如果字符串为空或长度为奇数，返回空字节数组；否则返回解析后的十六进制字节数组。
 */
fun String.hex(replace: Boolean = false): ByteArray {
    if (this.isEmpty() || this.length % 2 == 1) return ByteArray(0)
    var j_i = 0
    val j_src = if (replace) this.replace(" ", "").replace("\r", "").replace("\n", "").replace("\t", "").toByteArray() else this.toByteArray()
    val j_buf = ByteArray(j_src.size / 2)
    var j_src_i = 0
    while (j_src_i < j_src.size) {
        val j_left = reverseHexTable[j_src[j_src_i].toInt()]
        val j_right = reverseHexTable[j_src[j_src_i + 1].toInt()]
        if (j_left > 0x0f || j_right > 0x0f) return ByteArray(0)
        j_buf[j_i] = (j_left shl 4 or j_right).toByte()
        j_i++
        j_src_i += 2
    }
    return j_buf
}
