package cn.qqck.kotlin.tools

import kotlinx.coroutines.launch
import kotlinx.coroutines.runBlocking
import okhttp3.*
import java.security.KeyStore
import java.security.SecureRandom
import java.security.cert.X509Certificate
import java.time.Duration
import java.util.concurrent.TimeUnit
import javax.net.ssl.*

object Okhttp {
    private val ConnectionPool: ConnectionPool = ConnectionPool(0, 1, TimeUnit.MILLISECONDS) // 禁用连接池
    private val SSLSocketFactory: SSLSocketFactory
    private val X509TrustManager: X509TrustManager

    init {
        val j_sslct = SSLContext.getInstance("SSL")
        j_sslct.init(null, arrayOf<TrustManager>(object : X509TrustManager {
            override fun checkClientTrusted(chain: Array<X509Certificate>, authType: String) {
            }

            override fun checkServerTrusted(chain: Array<X509Certificate>, authType: String) {
            }

            override fun getAcceptedIssuers(): Array<X509Certificate> {
                return arrayOf()
            }
        }), SecureRandom())
        this.SSLSocketFactory = j_sslct.socketFactory

        val trustManagerFactory = TrustManagerFactory.getInstance(TrustManagerFactory.getDefaultAlgorithm())
        trustManagerFactory.init(null as KeyStore?)
        val trustManagers = trustManagerFactory.trustManagers
        check(!(trustManagers.size != 1 || trustManagers[0] !is X509TrustManager)) { "Unexpected default trust managers:" + trustManagers.contentToString() }
        this.X509TrustManager = trustManagers[0] as X509TrustManager
    }

    class Option {
        /**
         * 重定向，默认为 false
         */
        private var redirects = false

        /**
         * 重定向，默认为 false
         */
        fun redirects(): Boolean {
            return this.redirects
        }

        /**
         * 重定向，默认为 false
         */
        fun redirects(v: Boolean): Option {
            this.redirects = v
            return this
        }

        /**
         * SSL验证，默认为 false
         */
        private var SSLVerification = false

        /**
         * SSL验证，默认为 false
         */
        fun SSLVerification(): Boolean {
            return this.redirects
        }

        /**
         * SSL验证，默认为 false
         */
        fun SSLVerification(v: Boolean): Option {
            this.SSLVerification = v
            return this
        }

        /**
         * 请求超时时间，默认为 60s
         */
        private var Timeout = Duration.ofSeconds(60)

        /**
         * 请求超时时间，默认为 60s
         */
        fun Timeout(): Duration {
            return this.Timeout
        }

        /**
         * 请求超时时间，默认为 60s
         */
        fun Timeout(v: Duration): Option {
            this.Timeout = v
            return this
        }
    }

    data class Body(
        /**
         * @see Response.code
         */
        val code: Int,
        /**
         * @see Response.protocol
         */
        val protocol: String,
        /**
         * @see Response.message
         */
        val message: String,
        /**
         * @see Response.headers
         */
        val headers: Headers,
        /**
         * @see Response.body
         */
        val body: ByteArray?,
    ) {
        override fun toString(): String {
            return "Body(code=${code}, protocol='${protocol}', message='${message}', headers=${headers}, body=${body?.size})"
        }

        override fun equals(other: Any?): Boolean {
            if (other == null) return false
            if (this === other) return true
            if (javaClass != other.javaClass) return false
            other as Body
            if (code != other.code) return false
            if (protocol != other.protocol) return false
            if (message != other.message) return false
            if (headers != other.headers) return false
            if (body == null) return other.body == null
            if (other.body == null) return false
            return body.contentEquals(other.body)
        }

        override fun hashCode(): Int {
            var j_hash = code
            j_hash += protocol.hashCode()
            j_hash += message.hashCode()
            j_hash += headers.hashCode()
            j_hash += body?.hashCode() ?: 0
            return j_hash
        }
    }

    /**
     *
     * POST示例：
     * ```
     * Okhttp.call("http://www.baidu.com") {
     *     it.post("asd".toRequestBody("multipart/form-data".toMediaTypeOrNull()))
     * }
     * ```
     * 文件上传示例：
     * ```
     * Okhttp.call("http://www.baidu.com") {
     *     it.post(
     *         MultipartBody.Builder().setType(MultipartBody.FORM) // 上传的文件名称——参数不可省
     *             .addFormDataPart(
     *                 "file",
     *                 "adasdsa",
     *                 File("${filesDir}/config.json").asRequestBody("image/jpeg".toMediaTypeOrNull())
     *             )
     *             .addFormDataPart("参数2", "参数2的值")
     *             .addFormDataPart("参数3", "参数3的值")
     *             .build()
     *     )
     * }
     * ```
     */
    fun call(url: String, option: Option = Option(), pre: (build: Request.Builder) -> Unit = {}): Body? = runBlocking {
        val j_client = OkHttpClient.Builder()
        j_client.connectionPool(ConnectionPool)
        if (!option.redirects()) j_client.followSslRedirects(false).followRedirects(false) // 禁止重定向
        if (!option.SSLVerification()) j_client.sslSocketFactory(SSLSocketFactory, X509TrustManager).hostnameVerifier { _, _ -> true } // 忽略 ssl 证书
        j_client.callTimeout(option.Timeout())
        j_client.readTimeout(option.Timeout()) // 读取超时
        val j_req = Request.Builder().url(url)
        pre(j_req)
        val j_call = j_client.build().newCall(j_req.build())
        var j_body: Body? = null
        launch {
            try {
                j_body = this@Okhttp.call_response(j_call.execute())
            } catch (e: Exception) {
                // e.printStackTrace()
            }
        }.join()
        j_call.cancel()
        j_call.clone()
        return@runBlocking j_body
    }

    private fun call_response(r: Response): Body {
        val j_body = Body(r.code, r.protocol.toString(), r.message, r.headers, r.body?.bytes())
        r.close()
        return j_body
    }

    fun isNet(): Boolean {
        val j_op = Option().Timeout(Duration.ofSeconds(3))
        this.call("http://www.aliyun.com/111", j_op)?.also { if (it.code == 301) return true }
        this.call("http://www.baidu.com/111", j_op)?.also { if (it.code == 404) return true }
        return false
    }
}
