package cn.qqck.kotlin.tools

object Phones {
    private val data = this.javaClass.getResourceAsStream("/phones.dat")!!.readAllBytes().uncompress()
    private val prefDict = HashMap<String, Int>()
    private val addrArr = mutableListOf<Addr>()
    private val ispArr: List<String>

    data class Addr(val 省份: String, val 城市: String, val 邮编: String, val 区号: String, val 区划: String)

    init {
        // 来自：https://www.qqzeng.com/phone/
        val prefSize = data.toInt()
        val descLength = data.toInt(8)
        val ispLength = data.toInt(12)

        val headLength = 20

        val startIndex = headLength + descLength + ispLength
        // 内容数组
        String(data, headLength, descLength).split("&").forEach { it ->
            if (it.isEmpty()) return@forEach
            val j_info = it.split("|")
            // 河南|郑州|450000|0371|410100
            if (j_info.size != 5) return@forEach
            addrArr.add(Addr(j_info[0], j_info[1], j_info[2], j_info[3], j_info[4]))
        }
        // 运营商数组
        ispArr = String(data, headLength + descLength, ispLength).split("&")
        for (m in 0..<prefSize) {
            val i: Int = m * 5 + startIndex
            val pref = data[i].toInt() and 0xFF
            val index = data.toInt(i + 1)
            prefDict[pref.toString()] = index
        }
    }

    data class Info(val 归属地: Addr, val 运营商: String)

    /**
     * 根据提供的手机号获取对应的归属地信息及运营商信息。
     *
     * @param value 输入的查询手机号，需满足长度至少为7。
     * @return 如果查询成功，返回包含归属地及运营商信息的 [Info] 对象；否则返回 null。
     */
    operator fun get(value: String): Info? {
        if (value.length < 7) return null
        val j_i = prefDict[value.substring(0, 3)]?.let {
            data.toShort(it + value.substring(3, 7).toInt() * 2).toInt()
        } ?: return null
        return Info(addrArr[j_i shr 5], ispArr[j_i and 0x001F])
    }
}
