package cn.qqck.kotlin.tools

val Random = kotlin.random.Random(System.nanoTime())

object Rand {

    /**
     * 生成一个指定范围内的随机整数。
     *
     * @param min 随机数的最小值，包含在范围内。
     * @param max 随机数的最大值，包含在范围内。
     * @return 指定范围内的随机整数。
     */
    fun int(min: Int, max: Int): Int = Random.nextInt(max) % (max - min + 1) + min

    /**
     * 生成指定范围内的随机Long类型数值。
     *
     * @param min 随机数的最小值（包含）。
     * @param max 随机数的最大值（包含）。
     * @return 一个位于[min, max]范围内的随机Long值。
     */
    fun long(min: Long, max: Long): Long = Random.nextLong(max) % (max - min + 1) + min

    /**
     * 生成指定大小的随机字节数组。
     *
     * @param size 随机字节数组的大小，必须是非负整数。
     * @return 生成的随机字节数组，其长度等于指定的大小。
     */
    fun bytes(size: Int): ByteArray = Random.nextBytes(size)

    object str {
        fun az(size: Int, ex: CharArray? = null): String {
            val j_str = CharArray(size)
            if (ex == null || ex.isEmpty()) {
                for (j_i in j_str.indices) j_str[j_i] = Random.nextInt(97, 123).toChar() // 123 是 'z' 的下一个字符
            } else {
                val j_ex_size = ex.size
                for (j_i in j_str.indices) j_str[j_i] = when (Random.nextInt(2)) {
                    0 -> Random.nextInt(97, 123).toChar() // 123 是 'z' 的下一个字符
                    else -> ex[Random.nextInt(0, j_ex_size)]
                }
            }
            return String(j_str)
        }

        fun AZ(size: Int, ex: CharArray? = null): String {
            val j_str = CharArray(size)
            if (ex == null || ex.isEmpty()) {
                for (j_i in j_str.indices) j_str[j_i] = Random.nextInt(65, 91).toChar() // 91 是 'Z' 的下一个字符
            } else {
                val j_ex_size = ex.size
                for (j_i in j_str.indices) j_str[j_i] = when (Random.nextInt(2)) {
                    0 -> Random.nextInt(65, 91).toChar() // 91 是 'Z' 的下一个字符
                    else -> ex[Random.nextInt(0, j_ex_size)]
                }
            }
            return String(j_str)
        }

        fun aZ(size: Int, ex: CharArray? = null): String {
            val j_str = CharArray(size)
            if (ex == null || ex.isEmpty()) {
                for (j_i in j_str.indices) j_str[j_i] = when (Random.nextInt(2)) {
                    0 -> Random.nextInt(97, 123).toChar() // 123 是 'z' 的下一个字符
                    else -> Random.nextInt(65, 91).toChar() // 91 是 'Z' 的下一个字符
                }
            } else {
                val j_ex_size = ex.size
                for (j_i in j_str.indices) j_str[j_i] = when (Random.nextInt(3)) {
                    0 -> Random.nextInt(97, 123).toChar() // 123 是 'z' 的下一个字符
                    1 -> Random.nextInt(65, 91).toChar() // 91 是 'Z' 的下一个字符
                    else -> ex[Random.nextInt(0, j_ex_size)]
                }
            }
            return String(j_str)
        }

        fun num(size: Int, ex: CharArray? = null): String {
            val j_str = CharArray(size)
            if (ex == null || ex.isEmpty()) {
                for (j_i in j_str.indices) j_str[j_i] = Random.nextInt(48, 58).toChar() // 58 是 '9' 的下一个字符
            } else {
                val j_ex_size = ex.size
                for (j_i in j_str.indices) j_str[j_i] = when (Random.nextInt(2)) {
                    0 -> Random.nextInt(48, 58).toChar() // 58 是 '9' 的下一个字符
                    else -> ex[Random.nextInt(0, j_ex_size)]
                }
            }
            return String(j_str)
        }

        fun num_az(size: Int, ex: CharArray? = null): String {
            val j_str = CharArray(size)
            if (ex == null || ex.isEmpty()) {
                for (j_i in j_str.indices) j_str[j_i] = when (Random.nextInt(2)) {
                    0 -> Random.nextInt(48, 58).toChar() // 58 是 '9' 的下一个字符
                    else -> Random.nextInt(97, 123).toChar() // 123 是 'z' 的下一个字符
                }
            } else {
                val j_ex_size = ex.size
                for (j_i in j_str.indices) j_str[j_i] = when (Random.nextInt(3)) {
                    0 -> Random.nextInt(48, 58).toChar() // 58 是 '9' 的下一个字符
                    1 -> Random.nextInt(97, 123).toChar() // 123 是 'z' 的下一个字符
                    else -> ex[Random.nextInt(0, j_ex_size)]
                }
            }
            return String(j_str)
        }

        fun num_AZ(size: Int, ex: CharArray? = null): String {
            val j_str = CharArray(size)
            if (ex == null || ex.isEmpty()) {
                for (j_i in j_str.indices) j_str[j_i] = when (Random.nextInt(2)) {
                    0 -> Random.nextInt(48, 58).toChar() // 58 是 '9' 的下一个字符
                    else -> Random.nextInt(65, 91).toChar()  // 91 是 'Z' 的下一个字符
                }
            } else {
                val j_ex_size = ex.size
                for (j_i in j_str.indices) j_str[j_i] = when (Random.nextInt(3)) {
                    0 -> Random.nextInt(48, 58).toChar() // 58 是 '9' 的下一个字符
                    1 -> Random.nextInt(65, 91).toChar()  // 91 是 'Z' 的下一个字符
                    else -> ex[Random.nextInt(0, j_ex_size)]
                }
            }
            return String(j_str)
        }

        fun num_aZ(size: Int, ex: CharArray? = null): String {
            val j_str = CharArray(size)
            if (ex == null || ex.isEmpty()) {
                for (j_i in j_str.indices) j_str[j_i] = when (Random.nextInt(3)) {
                    0 -> Random.nextInt(48, 58).toChar() // 58 是 '9' 的下一个字符
                    1 -> Random.nextInt(97, 123).toChar() // 123 是 'z' 的下一个字符
                    else -> Random.nextInt(65, 91).toChar()  // 91 是 'Z' 的下一个字符
                }
            } else {
                val j_ex_size = ex.size
                for (j_i in j_str.indices) j_str[j_i] = when (Random.nextInt(4)) {
                    0 -> Random.nextInt(48, 58).toChar() // 58 是 '9' 的下一个字符
                    1 -> Random.nextInt(97, 123).toChar() // 123 是 'z' 的下一个字符
                    2 -> Random.nextInt(65, 91).toChar()  // 91 是 'Z' 的下一个字符
                    else -> ex[Random.nextInt(0, j_ex_size)]
                }
            }
            return String(j_str)
        }

        fun ascii(size: Int, ex: CharArray? = null): String {
            val j_str = CharArray(size)
            if (ex == null || ex.isEmpty()) {
                for (j_i in j_str.indices) j_str[j_i] = Random.nextInt(33, 127).toChar() // 所有单字节字符
            } else {
                val j_ex_size = ex.size
                for (j_i in j_str.indices) j_str[j_i] = when (Random.nextInt(2)) {
                    0 -> Random.nextInt(33, 127).toChar() // 所有单字节字符
                    else -> ex[Random.nextInt(0, j_ex_size)]
                }
            }
            return String(j_str)
        }

        fun china(size: Int, ex: CharArray? = null): String {
            val j_str = CharArray(size)
            if (ex == null || ex.isEmpty()) {
                // java内部使用unicode编码，汉字的Unicode编码范围为\u4E00-\u9FCB \uF900-\uFA2D,如果不在这个范围内就不是汉字。
                for (j_i in j_str.indices) j_str[j_i] = Random.nextInt(0x4E00, 0x9FCB + 1).toChar()
            } else {
                val j_ex_size = ex.size
                for (j_i in j_str.indices) j_str[j_i] = when (Random.nextInt(2)) {
                    // java内部使用unicode编码，汉字的Unicode编码范围为\u4E00-\u9FCB \uF900-\uFA2D,如果不在这个范围内就不是汉字。
                    0 -> Random.nextInt(0x4E00, 0x9FCB + 1).toChar()
                    else -> ex[Random.nextInt(0, j_ex_size)]
                }
            }
            return String(j_str)
        }

    }

    private val _hexLittle = "0123456789abcdef".toByteArray()
    private val _hexBig = "0123456789ABCDEF".toByteArray()

    /**
     * 生成随机 mac
     * @param big 是否大写 mac
     */
    fun mac(big: Boolean = false): String {
        val j_buf = ByteArray(17)
        if (big) {
            j_buf[0] = _hexBig[Random.nextInt(16)]
            j_buf[1] = _hexBig[Random.nextInt(16)]
            j_buf[2] = 58
            j_buf[3] = _hexBig[Random.nextInt(16)]
            j_buf[4] = _hexBig[Random.nextInt(16)]
            j_buf[5] = 58
            j_buf[6] = _hexBig[Random.nextInt(16)]
            j_buf[7] = _hexBig[Random.nextInt(16)]
            j_buf[8] = 58
            j_buf[9] = _hexBig[Random.nextInt(16)]
            j_buf[10] = _hexBig[Random.nextInt(16)]
            j_buf[11] = 58
            j_buf[12] = _hexBig[Random.nextInt(16)]
            j_buf[13] = _hexBig[Random.nextInt(16)]
            j_buf[14] = 58
            j_buf[15] = _hexBig[Random.nextInt(16)]
            j_buf[16] = _hexBig[Random.nextInt(16)]
        } else {
            j_buf[0] = _hexLittle[Random.nextInt(16)]
            j_buf[1] = _hexLittle[Random.nextInt(16)]
            j_buf[2] = 58
            j_buf[3] = _hexLittle[Random.nextInt(16)]
            j_buf[4] = _hexLittle[Random.nextInt(16)]
            j_buf[5] = 58
            j_buf[6] = _hexLittle[Random.nextInt(16)]
            j_buf[7] = _hexLittle[Random.nextInt(16)]
            j_buf[8] = 58
            j_buf[9] = _hexLittle[Random.nextInt(16)]
            j_buf[10] = _hexLittle[Random.nextInt(16)]
            j_buf[11] = 58
            j_buf[12] = _hexLittle[Random.nextInt(16)]
            j_buf[13] = _hexLittle[Random.nextInt(16)]
            j_buf[14] = 58
            j_buf[15] = _hexLittle[Random.nextInt(16)]
            j_buf[16] = _hexLittle[Random.nextInt(16)]
        }
        return String(j_buf)
    }

    /**
     * 生成随机 androidId
     * @param big 是否大写 androidId
     */
    fun androidId(big: Boolean = false): String {
        val j_buf = ByteArray(16)
        if (big) {
            j_buf[0] = _hexBig[Random.nextInt(16)]
            j_buf[1] = _hexBig[Random.nextInt(16)]
            j_buf[2] = _hexBig[Random.nextInt(16)]
            j_buf[3] = _hexBig[Random.nextInt(16)]
            j_buf[4] = _hexBig[Random.nextInt(16)]
            j_buf[5] = _hexBig[Random.nextInt(16)]
            j_buf[6] = _hexBig[Random.nextInt(16)]
            j_buf[7] = _hexBig[Random.nextInt(16)]
            j_buf[8] = _hexBig[Random.nextInt(16)]
            j_buf[9] = _hexBig[Random.nextInt(16)]
            j_buf[10] = _hexBig[Random.nextInt(16)]
            j_buf[11] = _hexBig[Random.nextInt(16)]
            j_buf[12] = _hexBig[Random.nextInt(16)]
            j_buf[13] = _hexBig[Random.nextInt(16)]
            j_buf[14] = _hexBig[Random.nextInt(16)]
            j_buf[15] = _hexBig[Random.nextInt(16)]
        } else {
            j_buf[0] = _hexLittle[Random.nextInt(16)]
            j_buf[1] = _hexLittle[Random.nextInt(16)]
            j_buf[2] = _hexLittle[Random.nextInt(16)]
            j_buf[3] = _hexLittle[Random.nextInt(16)]
            j_buf[4] = _hexLittle[Random.nextInt(16)]
            j_buf[5] = _hexLittle[Random.nextInt(16)]
            j_buf[6] = _hexLittle[Random.nextInt(16)]
            j_buf[7] = _hexLittle[Random.nextInt(16)]
            j_buf[8] = _hexLittle[Random.nextInt(16)]
            j_buf[9] = _hexLittle[Random.nextInt(16)]
            j_buf[10] = _hexLittle[Random.nextInt(16)]
            j_buf[11] = _hexLittle[Random.nextInt(16)]
            j_buf[12] = _hexLittle[Random.nextInt(16)]
            j_buf[13] = _hexLittle[Random.nextInt(16)]
            j_buf[14] = _hexLittle[Random.nextInt(16)]
            j_buf[15] = _hexLittle[Random.nextInt(16)]
        }
        return String(j_buf)
    }

    /**
     * 随机生成 imei
     *
     * @param pre 指定起始部分 imei
     */
    fun imei(pre: String = ""): String {
        var j_sum = 0 // the control j_sum of digits
        val j_final = ByteArray(15)

        // 计算前面几位
        for (j_i in pre.indices) {
            var j_toAdd = pre[j_i].code - 48
            j_final[j_i] = _hexLittle[j_toAdd]
            if ((j_i + 1) % 2 == 0) { // special proc for every 2nd one
                j_toAdd *= 2
                if (j_toAdd >= 10) j_toAdd = (j_toAdd % 10) + 1
            }
            j_sum += j_toAdd // and even add them here!
        }

        // 生成后面几位
        for (j_i in pre.length..<14) { // generating all the base digits
            var j_toAdd = if (j_i == 0) Random.nextInt(9) + 1 else Random.nextInt(10)
            j_final[j_i] = _hexLittle[j_toAdd]
            if ((j_i + 1) % 2 == 0) { // special proc for every 2nd one
                j_toAdd *= 2
                if (j_toAdd >= 10) j_toAdd = (j_toAdd % 10) + 1
            }
            j_sum += j_toAdd // and even add them here!
        }
        j_final[14] = _hexLittle[(j_sum * 9) % 10] // calculating the control digit
        return String(j_final)
    }
}
