package dev.botta.json

import dev.botta.json.parser.JsonParser
import dev.botta.json.values.*
import java.io.Reader
import java.util.Base64

object Json {
    val NULL: JsonValue = JsonLiteral.NULL
    val TRUE: JsonValue = JsonLiteral.TRUE
    val FALSE: JsonValue = JsonLiteral.FALSE

    fun value(value: Any?): JsonValue {
        when (value) {
            null -> return this.NULL
            is JsonValue -> return value
            is Int -> return this.value(value)
            is Long -> return this.value(value)
            is Float -> return this.value(value)
            is Double -> return this.value(value)
            is Boolean -> return this.value(value)
            is String -> return this.value(value)
            is ByteArray -> return this.value(value)
            is List<*> -> return JsonArray(value.map { this.value(it) })
            is Map<*, *> -> {
                val json = JsonObject()
                value.forEach { (key, value) ->
                    json[key as String] = this.value(value)
                }
                return json
            }

            else -> return this.value("${value}")
        }
    }

    fun value(value: Int) = JsonNumber(value.toString(10))

    fun value(value: Long) = JsonNumber(value.toString(10))

    fun value(value: Float): JsonValue {
        require(!value.isInfinite() && !value.isNaN()) { "Infinite and NaN values not permitted in JSON" }
        return JsonNumber(this.cutOffPointZero(value.toString()))
    }

    fun value(value: Double): JsonValue {
        require(!value.isInfinite() && !value.isNaN()) { "Infinite and NaN values not permitted in JSON" }
        return JsonNumber(this.cutOffPointZero(value.toString()))
    }

    fun value(value: Boolean) = if (value) this.TRUE else this.FALSE

    fun value(string: String?) = string?.let { JsonString(it) } ?: this.NULL
    fun value(byteArray: ByteArray?) = byteArray?.let { JsonString(Base64.getEncoder().encodeToString(it)) } ?: this.NULL

    fun array() = JsonArray()

    fun array(values: List<Any?>): JsonArray {
        val array = JsonArray()
        values.forEach { array.add(this.value(it)) }
        return array
    }

    fun array(vararg values: Int) = this.array(values.toList())

    fun array(vararg values: Long) = this.array(values.toList())

    fun array(vararg values: Float) = this.array(values.toList())

    fun array(vararg values: Double) = this.array(values.toList())

    fun array(vararg values: Boolean) = this.array(values.toList())

    fun array(vararg values: String?) = this.array(values.toList())

    fun array(vararg values: JsonValue) = this.array(values.toList())

    fun array(vararg values: Any?) = this.array(values.toList())

    fun obj() = JsonObject()

    fun obj(pairs: List<Pair<String, Any?>>) = JsonObject(pairs)

    fun obj(vararg pairs: Pair<String, Any?>) = JsonObject(pairs.toList())

    fun parse(string: String) = JsonParser().parse(string)

    fun parse(reader: Reader) = JsonParser().parse(reader)

    private fun cutOffPointZero(string: String): String {
        return if (string.endsWith(".0")) {
            string.substring(0, string.length - 2)
        } else string
    }
}
